---
title: Troubleshoot
---

## Where do I report `abra` bugs / feature requests?

You can use [this issue tracker](https://git.coopcloud.tech/coop-cloud/organising/issues/new/choose).

## SSH connection issues?

When you run `abra server add <host>`, `abra` will read from your `~/.ssh/config` and try to match a `Host <host>` entry. If you can `ssh <host>` then you should be able to `abra server add <host>`.

For example, if you do `abra server add example.com`, you should have a matching entry that looks like this:

```
Host example.com
  Hostname example.com
  User exampleUser
  Port 12345
  IdentityFile ~/.ssh/example@somewhere
```

and your IdentityFile should be added to the authentication agent:

```
ssh-add ~/.ssh/example@somewhere
```

## "abra server ls" shows the wrong details?

You can use `abra server rm` to remove the incorrect details. Make sure to take a backup of your `~/.abra/servers/<domain>` first. You can then try to re-create by using `abra server add ...` again.

However, if you have Docker installed on the same machine you have `abra`, then there might be some confusion. If you run `docker context ls` you'll see that Docker uses context connection strings also. `abra` simply uses this approach. Sometimes, your Docker defined context details & your `abra` context details can get out of sync. You can use `docker context rm` to resolve this.

If you need to create a new context from Docker, you can do:

```
docker context create <domain> --docker "host=ssh://<user>@<domain>:<port>"
```

This is what we used to before we wrote `abra` to make it more convenient.

## Command-line flag handling is weird?

Unfortunately, there is a limitation in our underlying command-line library implementation for `abra` ([ref](https://github.com/urfave/cli/issues/1113)) (and more fundamentally in the design of flags in the Go programming language itself ([ref](https://utcc.utoronto.ca/~cks/space/blog/programming/GoFlagUIImportance))). We're aiming to work with upstream to resolve the flag handling but this it is not yet clear when this will be resolved.

Currently, the following example of flexible flag usage is supported:

```
abra app new gitea -S  # generate secrets, after args
abra app new -S gitea  # generate secrets, before args
```

But something like the following does not work as expected:

```
abra app new -S gitea -p
```

Where the position of flags is mixed before & after args. `-p` is ignored :cry:

We're still waiting for upstream patch which resovles this.

## Why can't `abra` support multiline in `.env` files?

We're sorry, it's an issue with an upstream dependency. See [`#291`](https://git.coopcloud.tech/coop-cloud/organising/issues/291) for more.

## I need some feature from the old deprecated bash abra?

There is an archive of the [old code here](https://git.coopcloud.tech/coop-cloud/abra-bash).

You can install it alongside the [supported version of Abra](https://git.coopcloud.tech/coop-cloud/abra) by using these commands:

```bash
git clone https://git.coopcloud.tech/coop-cloud/abra-bash ~/.abra/bash-src
ln -s ~/.abra/bash-src/abra ~/.local/bin/babra
```

## "Network not found" when deploying?

This appears to be an upstream issue for which we can't do much in `abra` to solve. See [`coop-cloud/organising#420`](https://git.coopcloud.tech/coop-cloud/organising/issues/420) for more info. The work-around is to leave more time in between undeploy/deploy operations so the runtime can catch up.

## Caller path in debug stacktrace doesn't exist

Debug stacktrace currently begins with `/drone/` due to CI. Remove the initial `/drone/` and the path is relative to the abra project root.
